from static_data.models import UserType,Vat,DocumentType, DocNumberingSeries, PaymentMethod, TransferPurpose, TransferMethod, MeansOfTransport, MeasurementUnits
from django.db import models
from django.core.signals import request_finished
from django.dispatch import receiver
from django.db.models.signals import post_save
from django.contrib.auth.models import User
from django.db.models import Sum




class InvoiceCategories(models.Model):
    """
    Invoice Services & Products
    """
    MODE = (
        ('SERVICES', 'SERVICES'),
        ('PRODUCTS', 'PRODUCTS')
    )
    code = models.CharField(max_length=50, blank=True, null=True, help_text="Code")
    description = models.CharField(max_length=200, blank=True, null=True, help_text="Descripion")
    measurement_unit = models.ForeignKey(MeasurementUnits, on_delete=models.SET_NULL,blank=True, null=True, help_text="Measurement Unit")
    price = models.FloatField(blank=True, null=True, help_text="Price")
    vat_category = models.ForeignKey(Vat, on_delete=models.SET_NULL, blank=True, null=True, help_text="Vat Category")
    exclusive_for = models.ForeignKey(UserType, on_delete=models.SET_NULL, blank=True, null=True, help_text="User Type")
    mode = models.CharField(choices=MODE, max_length=20,blank=True, null=True, help_text="Mode")

    is_deleted = models.BooleanField(default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return f'{self.pk}-{self.code}'

    class Meta:
        verbose_name_plural = "Invoices Categories"
        db_table = "invoice_categories"




class InvoiceDocument(models.Model):
    """
    Invoice Document Model
    """
    total_disc_percent = models.FloatField(default=0, blank=True, null=True, help_text="Discount Percent")
    total_disc_price = models.FloatField(blank=True, null=True, help_text="discount price")
    total_net_value = models.FloatField(blank=True, null=True, help_text="Net Value")
    total_vat_value = models.FloatField(blank=True, null=True, help_text="Vat Value")
    total_value = models.FloatField(blank=True, null=True, help_text="Total Value")
    pdf = models.FileField(blank=True, null=True, help_text="Total Value")

    document_type = models.ForeignKey(DocumentType, on_delete=models.CASCADE,blank=True, null=True, help_text="Document Type")
    doc_numbering_series = models.ForeignKey(DocNumberingSeries, blank=True, null=True, on_delete=models.CASCADE, help_text="Document Numbering Series")
    number = models.IntegerField(blank=True, null=True, help_text="Number")
    date = models.DateField(blank=True, null=True, help_text="Date")
    user_code = models.CharField(max_length=199, blank=True, null=True, help_text="User Code")
    buisness_name = models.CharField(max_length=199, blank=True, null=True, help_text="Buisness Name")
    address = models.CharField(max_length=199, blank=True, null=True, help_text="Address")
    payment_method = models.ForeignKey(PaymentMethod, on_delete=models.CASCADE, blank=True, null=True, help_text="Payment Method")
    transfer_purpose = models.ForeignKey(TransferPurpose, on_delete=models.CASCADE, blank=True, null=True, help_text="Transfer Purpose")
    transfer_method = models.ForeignKey(TransferMethod, on_delete=models.CASCADE, blank=True, null=True, help_text="Transfer Method")
    means_of_transport = models.ForeignKey(MeansOfTransport, on_delete=models.CASCADE, blank=True, null=True, help_text="Means of transport")

    is_active = models.BooleanField(default=False, blank=True, null=True, help_text="If the Invoice is active")
    is_deleted = models.BooleanField(default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    class Meta:
        verbose_name_plural = "Invoice Documents"
        db_table = "invoice_document"


class InvoiceItems(models.Model):
    """
    Invoice Item Model
    """
    invoice_document = models.ForeignKey(InvoiceDocument, on_delete=models.CASCADE, blank=True, null=True, help_text="Invoice Document")
    invoice_category = models.ForeignKey(InvoiceCategories, on_delete=models.CASCADE, blank=True, null=True, help_text="Invoice Category")

    quantity = models.IntegerField(blank=True, null=True,default=0, help_text="Quantity")
    price = models.FloatField(blank=True, null=True,default=0, help_text="price")
    disc_percent = models.FloatField(default=0, blank=True, null=True, help_text="Discount Percent")
    disc_price = models.FloatField(blank=True, null=True,default=0, help_text="discount price")
    net_value = models.FloatField(blank=True, null=True,default=0, help_text="Net Value")
    vat_value = models.FloatField(blank=True, null=True,default=0, help_text="Vat Value")
    total_value = models.FloatField(blank=True, null=True,default=0, help_text="Total Value")

    is_deleted = models.BooleanField(default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return f'{self.pk}-{self.invoice_document}'


    class Meta:
        verbose_name_plural = "Invoices Items"
        db_table = "invoice_item"




class InvoiceComment(models.Model):
    """
    Invoice Comments
    """
    invoice_document = models.ForeignKey(InvoiceDocument, on_delete=models.CASCADE, blank=True, null=True, help_text="Invoice Document")
    comment = models.CharField(max_length=100, blank=True, null=True, help_text="Comment")
    
    created_by = models.ForeignKey(User, on_delete=models.CASCADE, blank=True, null=True, help_text="Created By")
    is_deleted = models.BooleanField(default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)


    def __str__(self):
        return f'{self.pk}-{self.invoice_document}'


    class Meta:
        verbose_name_plural = "Invoices Comments"
        db_table = "invoice_comment"



@receiver(post_save, sender=InvoiceItems)
def my_callback(sender,created,instance, **kwargs):
    if instance.invoice_document:
        if created:
            instance.invoice_document.is_active = True
        
        invoice_items = InvoiceItems.objects.filter(is_deleted=False, invoice_document=instance.invoice_document).aggregate(
            total_disc_percent = Sum('disc_percent'), total_disc_price = Sum('disc_price'),
            total_value = Sum('total_value'),total_vat_value = Sum('vat_value'),
            total_net_value = Sum('net_value') )
        for key,value in invoice_items.items():
            setattr(instance.invoice_document, key, value)
            instance.invoice_document.save()
        print(invoice_items)
        print("Request finished!")