from rest_framework import serializers
from .models import *
from .functionalities import *
from venue_owners.models import VenueOwner
from venues.models import Venue


class InvoiceCategoriesSerializer(serializers.ModelSerializer):
    """
    Get POST Service and products
    """
    id = serializers.IntegerField(required=False, help_text="ID")
    class Meta:
        model = InvoiceCategories
        fields = '__all__'
        read_only_fields = ['created_at','is_deleted','mode']


class InvoiceItemSerializer(serializers.ModelSerializer):
    """
    Add Edit Invoice item Serializer
    """
    category_code = serializers.CharField(source="invoice_category.code", required=False,default="")
    category_description = serializers.CharField(source="invoice_category.description", required=False,default="")
    category_vat_category_code = serializers.CharField(source="invoice_category.vat_category.vat_code", required=False,default="")
    category_vat_category_description = serializers.CharField(source="invoice_category.vat_category.vat_description", required=False,default="")
    category_vat_category_standard = serializers.CharField(source="invoice_category.vat_category.vat_standard", required=False,default="")
    category_vat_category_reduced = serializers.CharField(source="invoice_category.vat_category.vat_reduced", required=False,default="")
    category_vat_category_exemption = serializers.CharField(source="invoice_category.vat_category.vat_exemption", required=False,default="")
    category_measurement_unit_code = serializers.CharField(source="invoice_category.measurement_unit.measurement_code", required=False,default="")
    category_measurement_unit_description = serializers.CharField(source="invoice_category.measurement_unit.measurement_description", required=False,default="")

    class Meta:
        model = InvoiceItems
        fields = '__all__'


class InvoiceDocumentSerializer(serializers.ModelSerializer):
    """
    Invoice Serializer
    """
    id = serializers.IntegerField(required=False, help_text="ID")
    item = serializers.JSONField(required=False, write_only=True, help_text='[{"quantity":1, "invoice_category":1, "price":12, "disc_percent":12, "disc_price":12, "net_value":12, "vat_value":12, "total_value":12}]')
    date_from = serializers.DateField(required=False, write_only=True)
    date_to = serializers.DateField(required=False, write_only=True)
    number_from = serializers.IntegerField(required=False, write_only=True)
    number_to = serializers.IntegerField(required=False, write_only=True)

    def validate_item(self, values):
        for value in values:
            if value.get("invoice_category", False):
                invoice_category_obj = InvoiceCategories.objects.filter(id=value['invoice_category'])
                if invoice_category_obj.exists():
                    value['invoice_category'] = invoice_category_obj.last()
                else:
                    raise serializers.ValidationError("Incorrect Invoice Category")
        return values

    def save(self):
        """
        Save Invoice Serializer
        """
        if "id" in self.validated_data:
            invoice_items = False
            if "item" in self.validated_data:
                invoice_items = self.validated_data.pop("item")
                InvoiceItems.objects.filter(invoice_document=self.instance).delete() 
                invoice_items = [InvoiceItems(
                    **i,
                    invoice_document=self.instance) for i in invoice_items
                ]
                InvoiceItems.objects.bulk_create(invoice_items)
            return UpdateInvoiceDocument(self.validated_data, self.instance)
        else:
            return InsertInvoiceDocument(self)

    class Meta:
        model = InvoiceDocument
        fields = '__all__'


class InvoiceDocumentDetailedSerializer(serializers.ModelSerializer):
    """
    Get Detailed invoice Document
    """
    invoice_items = serializers.SerializerMethodField("_get_invoice_items")

    def _get_invoice_items(self, obj):
        invoice_obj = InvoiceItems.objects.filter(invoice_document=obj, is_deleted=False)
        return InvoiceItemSerializer(invoice_obj, many=True).data
    class Meta:
        model = InvoiceDocument
        fields = '__all__'

class GetInvoiceCategoryFromCodeSerializer(serializers.ModelSerializer):
    """
    GetInvoiceCategoryFromCodeSerializer
    """
    class Meta:
        model = InvoiceCategories
        fields = '__all__'


class InvoiceCommentSerializer(serializers.ModelSerializer):
    """
    Serializer for invoice Comment Serializer
    """
    created_by_name = serializers.SerializerMethodField("_get_created_by_name", read_only=True)

    def _get_created_by_name(self, obj):
        created_by = obj.created_by
        return ""

    class Meta:
        model = InvoiceComment
        fields = '__all__'
    


class GetUserDataFromUserCodeSerializer(serializers.ModelSerializer):
    """
    Get All Data from User Code
    """
    buisness_name = serializers.SerializerMethodField("_get_buisness_name")
    address = serializers.SerializerMethodField("_get_address")
    address1 = serializers.SerializerMethodField("_get_address1")
    address2 = serializers.SerializerMethodField("_get_address2")

    def _get_buisness_name(self, obj):
        try:
            if self.Meta.model == VenueOwner:
                venue_obj = Venue.objects.filter(created_by=obj.user)
                if venue_obj.exists():
                    return venue_obj.last().business_name
            obj.buisness_name
        except:
            return ""

    def _get_address(self, obj):
        all_obj = list()
        obj_needed = ["street", "venue_number", "pincode", "city", "country"]
        try:
            for i in obj_needed:
                if getattr(obj, i) != None:
                    if i in ['city', 'country']:
                        all_obj.append(getattr(obj, i).name)
                    else:
                        all_obj.append(getattr(obj, i))
            # return obj.street + obj.venue_number + obj.pincode + obj.city.name + obj.country.name
            return ",".join(all_obj)
        except Exception as e:
            return ""
        



    def _get_address2(self, obj):
        try:
            return obj.address2
        except:
            return "" 
    
    def _get_address1(self, obj):
        try:
            return obj.address1
        except:
            return "" 
    class Meta:
        model = None
        fields = ['id','buisness_name', 'address','address1', "address2"]



class GetDataAutoCompleteSerializer(serializers.ModelSerializer):
    obj_id = serializers.SerializerMethodField("_get_id")

    def _get_id(self, obj):
        if self.Meta.model == Venue:
            print(obj)
            obj_id = VenueOwner.objects.filter(user=obj.created_by).last()
            return obj_id.id
        return obj.id

    class Meta:
        model = None
        fields = None

class GetInvoiceCategoryAutoComplete(serializers.ModelSerializer):
    class Meta:
        model = InvoiceCategories
        fields = ['code']