import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from message.serializers import *
from message.models import Notification
from venue_owners.models import VenueOwner
from players.models import Player
from professionals.models import Professional
from django.db.models import Q
from advertisers.models import Advertiser
from static_data.models import NotificationIcon

############################ -- GET -- ###############################


class DeleteMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Message
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = MessagesDeleteSerializer
    # Delete Message

    @transaction.atomic
    def post(self, request, format=None):

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            responsedata = []
            response = {}
            if serializer.data['MessageType'] == 'Inbox':
                if MessagesInbox.objects.filter(recipient=request.user, id=serializer.data['id'], is_deleted=False).exists():
                    data = MessagesInbox.objects.filter(
                        recipient=request.user, id=serializer.data['id'], is_deleted=False).first()
                    data.is_deleted = True
                    data.save(update_fields=['is_deleted'])
            elif serializer.data['MessageType'] == 'Sentbox':
                if MessagesInbox.objects.filter(sender=request.user, id=serializer.data['id'], is_deleted=False).exists():
                    data = MessagesInbox.objects.filter(
                        sender=request.user, id=serializer.data['id'], is_deleted=False).first()
                    data.is_deleted = True
                    data.save(update_fields=['is_deleted'])
            else:
                response["result"] = 0
                responsedata.append("Incorrect Message Type")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response['result'] = 1
            response['data'] = "Successfully deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MessageStorageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Inbox OR SentBOX Messages
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = MessagesInboxSerializer
    # GET Unread Messages

    @transaction.atomic
    def post(self, request, format=None):
        # Apply pagination
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")

            response = {}
            responsedata = []
            extra_data = {}

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            if 'MessageType' in serializer.validated_data:

                if serializer.data['MessageType'] == 'Inbox':

                    if 'from_date' in serializer.validated_data:
                        extra_data["created_at__gte"] = serializer.validated_data['from_date']
                    if 'to_date' in serializer.validated_data:
                        extra_data['created_at__lte'] = serializer.validated_data['to_date']
                    if 'is_read' in serializer.validated_data:
                        extra_data['is_read'] = serializer.validated_data['is_read']

                    UnreadMessage = MessagesInbox.objects.filter(
                        recipient=request.user, is_deleted=serializer.data['is_deleted'], **extra_data).order_by("-created_at")
                    unread_message_count = UnreadMessage.count()
                    UnreadMessage = UnreadMessage[start_data:end_data]

                elif serializer.data['MessageType'] == 'Sentbox':
                    unread_message_count = MessagesInbox.objects.filter(
                        sender=request.user, is_deleted=False).order_by("-created_at").count()
                    UnreadMessage = MessagesInbox.objects.filter(
                        sender=request.user, is_deleted=False).order_by("-created_at")[start_data:end_data]

                else:
                    response["result"] = 0
                    responsedata.append("Incorrect Type")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            else:
                # deleted Messages
                sent_deleted = MessagesInbox.objects.filter(
                    sender=request.user, **serializer.validated_data).order_by("-created_at")[start_data:end_data]

                sent_total_count = MessagesInbox.objects.filter(
                    sender=request.user, **serializer.validated_data).order_by("-created_at").count()

                inbox_deleted = MessagesInbox.objects.filter(
                    recipient=request.user, **serializer.validated_data).order_by("-created_at")[start_data:end_data]
                inbox_total_count = MessagesInbox.objects.filter(
                    recipient=request.user, **serializer.validated_data).order_by("-created_at").count()

                sent_serializer = MessagesInboxSerializer(
                    sent_deleted, many=True)
                inbox_serializer = MessagesInboxSerializer(
                    inbox_deleted, many=True)

                data = {}
                data['Sent_Items'] = sent_serializer.data
                data['Inbox_Items'] = inbox_serializer.data
                data['Total'] = sent_serializer.data + inbox_serializer.data
                response['result'] = 1
                response['data'] = data
                response['total_count'] = sent_total_count + inbox_total_count
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = MessagesInboxSerializer(UnreadMessage, many=True)

            response['result'] = 1
            response['data'] = serializer.data
            response['total_count'] = unread_message_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetMessageSerializer

    single_message_class = GetSingleMessageSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.validated_data['id'].is_read = True
            serializer.validated_data['id'].save()

            message_class = self.single_message_class(
                serializer.validated_data['id'], many=False)

            object_data = Notification.objects.filter(
                recipient=request.user).last()
            object_data.is_read = True
            object_data.save()

            response['result'] = 1
            responsedata.append('Successfull')
            response['data'] = message_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ComposeMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ComposeMessageSerialier
    Notification_serializer = NotificationsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if "recipient" not in serializer.validated_data:
                response["result"] = 0
                responsedata.append("Please enter valid Recipient username")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            if User.objects.filter(username=request.user).exists():
                serializer.validated_data['sender'] = request.user
                getID = serializer.save()
                message = MessagesInbox.objects.get(id=getID.id)

                # Get Message Icons
                notification_obj = NotificationIcon.objects.get(name="Message")

                # Notifications
                data = {"category": "Message", "button_text": "Reply", "icon": notification_obj, "subject": "You have a new Message from " +
                        str(request.user), "recipient": message.recipient, "content_object": message, 'body': serializer.validated_data['message']}

                data = Notification(**data)
                data.save()
                # Notification Ends

                response["result"] = 1
                response['errors'] = ["Successfully sent"]
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 0
                responsedata.append("No Authorized User")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_200_OK)


class GetMessageActionAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get Reply Forward Message
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetMessageActionSerializer

    forward_serializer_class = ForwardMessageSerializer
    reply_serializer_class = ReplyMessageSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if serializers.validated_data['action'] == "Forward":
                message_serializer = self.forward_serializer_class(
                    serializers.validated_data['id'], many=False)
            elif serializers.validated_data['action'] == "Reply":
                message_serializer = self.reply_serializer_class(
                    serializers.validated_data['id'], many=False)
            # read message
            message_obj = serializers.validated_data['id']
            message_obj.is_read = True
            message_obj.save()
            response['result'] = 1
            response['data'] = message_serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


# NOTIFICATION

class GetNotificationsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Notifications
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = NotificationPaginationSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        pagination = request.data['page']
        start_data = (int(pagination) - 1) * 10
        end_data = start_data + 10

        notification_count = Notification.objects.filter(
            recipient=request.user).order_by("-created_at").count()
        UnreadNotifications = Notification.objects.filter(
            recipient=request.user).order_by("-created_at")[start_data:end_data]
        serializer = NotificationsSerializer(UnreadNotifications, many=True)

        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = notification_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class NotificationReadAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    To mark all notification as read
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, format=None):

        response = {}
        responsedata = []
        unread_notification = Notification.objects.filter(
            is_read=False, recipient=request.user)
        for i in unread_notification:
            i.is_read = True
            i.save()
        responsedata.append("Successful")
        response['result'] = 1
        response['errors'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetNameEmailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Names and Emails
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetEmailSerializer

    venue_owner_class = VenueOwnerSerializer
    professional_class = ProfessionalSerializer
    advertiser_class = AdvertiserSerializer
    player_class = PlayerSerializer

    def post(self, request, format=None):

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            response = {}
            responsedata = []

            # Venue Owner
            data = VenueOwner.objects.filter(Q(email__icontains=serializers.validated_data['key_word']) | Q(
                first_name__contains=serializers.validated_data['key_word']) | Q(last_name__contains=serializers.validated_data['key_word']))
            venue_owner_data = self.venue_owner_class(data, many=True)

            # Professional
            data = Professional.objects.filter(Q(email__icontains=serializers.validated_data['key_word']) | Q(
                first_name__contains=serializers.validated_data['key_word']) | Q(last_name__contains=serializers.validated_data['key_word']))
            professional_data = self.professional_class(data, many=True)

            # Advertiser
            data = Advertiser.objects.filter(Q(email__icontains=serializers.validated_data['key_word']) | Q(
                first_name__contains=serializers.validated_data['key_word']) | Q(last_name__contains=serializers.validated_data['key_word']))
            advertiser_data = self.advertiser_class(data, many=True)

            # Player
            data = Player.objects.filter(Q(email__icontains=serializers.validated_data['key_word']) | Q(
                first_name__contains=serializers.validated_data['key_word']) | Q(last_name__contains=serializers.validated_data['key_word']))
            player_data = self.player_class(data, many=True)

            response['result'] = 1
            response['data'] = venue_owner_data.data + \
                professional_data.data + advertiser_data.data + player_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
