from django.db import models
import datetime
import binascii
import os
import random

from django.contrib.auth.models import User
from django.utils.encoding import python_2_unicode_compatible

# from internal_system.models import InternalUser


class PlatformApiCall(models.Model):
    """ Info about API requested by user """
    user              = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE)
    requested_url     = models.CharField(max_length=250, null=True, verbose_name='requested url')
    requested_data    = models.TextField(verbose_name='requested data')
    response_data     = models.TextField(verbose_name='response data')
    client_ip_address = models.CharField(max_length=200, null=True, blank=True)
    created_at        = models.DateTimeField(auto_now_add=True, null=True)

    def __str__(self):
        return '{}'.format(self.requested_url)

    class Meta:
        verbose_name_plural = 'Platform API Calls'
        db_table = 'platform_api_call'


class UserFcmObject(models.Model):
    """ Registered User's FCM details reside here """
    user          = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE)
    device_id     = models.CharField(blank=True, null=True, max_length=400, help_text='user\'s device id')
    fcm_id        = models.CharField(blank=True, null=True, max_length=400, help_text='user\'s fcm id')
    device_type   = models.CharField(blank=True, null=True, max_length=400, help_text='user\'s device type')
    created_at    = models.DateTimeField(auto_now_add=True, null=True)

    def __str__(self):
        return '{}-{}'.format(self.user, self.fcm_id)

    class Meta:
        verbose_name_plural = 'User FCM Ids'
        db_table = 'user_fcm'


class UserOtp(models.Model):
    """ OTPs reside here """
    user          = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE)
    otp           = models.CharField(blank=True, max_length=100, help_text='User\'s OTP')
    valid_till    = models.DateTimeField(blank=True, null=True, help_text='Time until when the OTP is valid')
    created_at    = models.DateTimeField(auto_now_add=True, null=True)
    is_active     = models.BooleanField(default=True,blank=True)
    updated_at = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        self.otp = ''.join([random.choice('1234567890') for i in range(6)])
        self.valid_till = (datetime.datetime.now() + datetime.timedelta(minutes=5))
        super(UserOtp, self).save(*args, **kwargs)

    def __str__(self):
        return '{} - {} - {}'.format(self.pk, self.user, self.otp)

    class Meta:
        verbose_name_plural = 'User OTPs'
        db_table = 'user_otp'


class SmsCentral(models.Model):
    """ info about sms message mobile verification otp code store here """
    mobile     = models.CharField(blank=True, null=True, default='', max_length=10)
    
    message    = models.CharField(blank=True, null=True, max_length=1000)
    url        = models.TextField(blank=True, null=True)
    response   = models.TextField(blank=True, null=True)
    
    user_otp   = models.ForeignKey(UserOtp,on_delete=models.CASCADE,blank=True,null=True) 
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.mobile

    class Meta:
        verbose_name_plural = "Platform SMS Central"
        db_table = "sms_central"


class UserToken(models.Model):
    """ custom authorization token model (for email verification) """
    PURPOSE = (
        ('EMAIL VERIFICATION', 'EMAIL VERIFICATION'),
        ('EMAIL CHANGE', 'EMAIL CHANGE'),
        ('RESET PASSWORD', 'RESET PASSWORD'),
    )

    user       = models.ForeignKey(User, on_delete=models.CASCADE, related_name='custom_token')
    purpose    = models.CharField(max_length=50, choices=PURPOSE, null=True, blank=True)  
    key        = models.CharField(max_length=40, primary_key=True)
    is_active  = models.BooleanField(default=True,blank=True,null=True)
    last_used  = models.DateTimeField(blank=True,null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(UserToken, self).save(*args, **kwargs)

    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()

    def __str__(self):
        return self.key

    class Meta:
        verbose_name_plural = 'Platform User Tokens'
        db_table            = 'platform_user_token'



# @python_2_unicode_compatible
# class UserEmailToken(models.Model):
#     """ custom authorization token model (for email verification) """
#     PURPOSE = (
#         ('EMAIL VERIFICATION', 'EMAIL VERIFICATION'),
#         ('EMAIL CHANGE', 'EMAIL CHANGE'),
#         ('RESET PASSWORD', 'RESET PASSWORD'),
#     )

#     key           = models.CharField(max_length=40, primary_key=True)
#     purpose       = models.CharField(max_length=50, choices=PURPOSE, null=True, blank=True)
#     internal_user = models.ForeignKey(InternalUser, on_delete=models.CASCADE, related_name='custom_token')
#     created_at    = models.DateTimeField(auto_now_add=True, blank=True, null=True)

#     def save(self, *args, **kwargs):
#         if not self.key:
#             self.key = self.generate_key()
#         return super(UserEmailToken, self).save(*args, **kwargs)

#     def generate_key(self):
#         return binascii.hexlify(os.urandom(20)).decode()

#     def __str__(self):
#         return self.key

#     class Meta:
#         verbose_name_plural = 'User - Email Tokens'
#         db_table            = 'user_email_token'




class RegisterPrivateKey(models.Model):
    """
    Register Private Keys
    """
    user = models.OneToOneField(User, on_delete=models.CASCADE, null=True, unique=True, blank=True, help_text="User for the Key")
    key = models.CharField(max_length=100, blank=True, null=True, help_text="Key of the User")
    is_active = models.BooleanField(default=True, help_text="Status of the key")
    activated_at = models.DateTimeField(blank=True, null=True, help_text="Time of activation")

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(RegisterPrivateKey, self).save(*args, **kwargs)


    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()


    def __str__(self):
        return '{} - {} - {}'.format(self.pk, self.user, self.key)

    class Meta:
        verbose_name_plural = "Registeration Key"
        db_table = 'registeration_key'
