from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import *
from venues.models import VenueField, VenueFieldService, Venue, VenueServiceCostQuantity, VenueServiceCost
from venue_owners.models import Schedule_Slot
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class Player(models.Model):
    """ Register Player details reside here """
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    first_name = models.CharField(
        max_length=100, null=True, help_text="user first name")
    middle_name = models.CharField(
        max_length=100, null=True, help_text="user middle name", blank=True)
    last_name = models.CharField(
        max_length=100, null=True, help_text="user last name", blank=True)
    nick_name = models.CharField(
        max_length=100, null=True, help_text="user nick name", blank=True)

    email = models.CharField(max_length=100, null=True, help_text="user email")
    new_email = models.CharField(
        max_length=100, null=True, blank=True, help_text="user new email")
    mobile = models.CharField(max_length=15, null=True,
                              help_text="user mobile number", unique=True)
    new_mobile = models.CharField(
        max_length=15, null=True, help_text="user new mobile number", blank=True)
    gender = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES,
                              help_text="M - Male, F - Female, O - Other")

    street = models.CharField(max_length=150, blank=True,
                              null=True, help_text="Street & Number, P.O.box c/o")
    latitude = models.FloatField(blank=True, null=True, help_text="Latitude")
    longitude = models.FloatField(blank=True, null=True, help_text="Longitude")
    # city = models.CharField(max_length=50, blank=True,null=True, help_text="city")

    # country = models.CharField(max_length=50, blank=True, null=True, help_text="country")

    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)

    pincode = models.CharField(
        max_length=10, blank=True, null=True, help_text="code")
    image = models.ImageField(storage=S3CustomStorage(
    ), upload_to="player_images", blank=True, null=True)
    dob = models.DateField(
        blank=True, null=True, help_text='date of birth format YYYY-MM-DD', verbose_name='date of birth')

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")
    is_email_verified = models.BooleanField(default=False)

    # created_by = models.ForeignKey(InternalUser, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.pk, self.mobile, self.first_name)

    class Meta:
        unique_together = ('user', 'email')
        verbose_name_plural = "Players"
        db_table = 'player'


class PlayerAvailability(models.Model):
    """
    Player Availability Data
    """

    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)

    date = models.DateField(blank=True, null=True, help_text="YYYY-MM-DD")
    start_time = models.TimeField(
        blank=True, null=True, help_text="HH:MM 24hrs")
    end_time = models.TimeField(blank=True, null=True, help_text="HH:MM 24hrs")

    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True, help_text="Public Game")

    public_game = models.BooleanField(
        default=False, blank=True, help_text="Public Game")
    private_game = models.BooleanField(
        default=False, blank=True, help_text="Private Game")

    note = models.TextField(blank=True, null=True, help_text="Note")

    is_deleted = models.BooleanField(default=False, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{} {} {} {}".format(self.player, self.date, self.start_time, self.end_time)

    class Meta:
        verbose_name_plural = 'Player Availability'
        db_table = 'player_availability'


class PlayerRating(models.Model):
    """
    Player Ratings Data
    """
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)

    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    rating = models.IntegerField(default=0)

    is_deleted = models.BooleanField(default=False, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{} {} {} {}".format(self.player, self.date, self.start_time, self.end_time)

    class Meta:
        verbose_name_plural = 'Player Rating'
        db_table = 'player_rating'


class PlayerSkills(models.Model):
    """
    Player Skills Data
    """
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    position = models.ForeignKey(
        Position, on_delete=models.CASCADE, blank=True, null=True)

    level = models.ForeignKey(PlayerLevel,on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(default=False, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{} {} {} {}".format(self.player, self.sport, self.position, self.created_at)

    class Meta:
        verbose_name_plural = 'Player Skills'
        db_table = 'player_skills'


class FriendList(models.Model):
    """
    Player Friend Lists Data
    """
    user = models.ForeignKey(
        Player, on_delete=models.CASCADE, related_name="MainUser")
    contacts = models.ManyToManyField(Player)

    def __str__(self):
        return "{}".format(self.user)

    class Meta:
        verbose_name_plural = 'Player Contacts'
        db_table = 'player_contacts'


class NewsLetter(models.Model):
    """
    Player Newsletter Subscriptions Data
    """
    email = models.CharField(max_length=50, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.email)

    class Meta:
        verbose_name_plural = "News Letter"
        db_table = "news_letter"


class TeamDetails(models.Model):
    """
    Player Create Team Step 1
    """
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)
    team_name = models.TextField(max_length=100, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)

    # country = models.CharField(max_length=100, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    perfecture = models.CharField(max_length=10, blank=True, null=True, help_text="Perfecture")
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)

    # city = models.CharField(max_length=100, blank=True, null=True)
    region = models.CharField(max_length=100, blank=True, null=True)
    team_level = models.ManyToManyField(TeamLevel, blank=True)

    date_founded = models.DateField(blank=True, null=True)
    team_image = models.ImageField(
        upload_to="team_images", blank=True, null=True)
    admin_first_name = models.CharField(max_length=100, blank=True, null=True, help_text="Administrator First name")
    admin_last_name = models.CharField(max_length=100, blank=True, null=True, help_text="Administrator First name")
    player_count = models.IntegerField(default=0, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.team_name, self.sport, self.country)

    class Meta:
        verbose_name_plural = "Team Details"
        db_table = 'team_details'


class TeamContacts(models.Model):
    """
    Player Team Contacts
    """
    team_details = models.ForeignKey(
        TeamDetails, on_delete=models.CASCADE, blank=True, null=True)

    first_name = models.TextField(max_length=100, blank=True, null=True)
    last_name = models.TextField(max_length=100, blank=True, null=True)

    mobile = models.TextField(max_length=14, blank=True, null=True)
    email = models.TextField(max_length=50, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.team_details, self.first_name, self.last_name)

    class Meta:
        verbose_name_plural = "Team Contacts"
        db_table = "team_contact"


class TeamMember(models.Model):
    """
    Models for Team members
    """
    STATUS_CHOICE = (
        ("Pending", "Pending"),
        ("Approved", "Approved"),
        ("Declined", "Declined")
    )

    team_details = models.ForeignKey(
        TeamDetails, on_delete=models.CASCADE, blank=True, null=True)
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)

    email = models.CharField(max_length=100, blank=True,
                             null=True, help_text="Email address")
    first_name = models.CharField(
        max_length=100, blank=True, null=True, help_text="First Name")

    last_name = models.CharField(
        max_length=100, blank=True, null=True, help_text="last Name")
    position = models.ForeignKey(
        Position, on_delete=models.CASCADE, blank=True, null=True, help_text="Player Position")

    status = models.CharField(max_length=20, choices=STATUS_CHOICE,
                              default=STATUS_CHOICE[0][0], blank=True, null=True, help_text="Status of the player")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="Deleted Status")
    is_owner = models.BooleanField(
        default=False, blank=True, null=True, help_text="Owner of the team")

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.team_details, self.player, self.status)

    class Meta:
        verbose_name_plural = "Team Members"
        db_table = "team_member"


class TeamCalendar(models.Model):
    """
    Player Team  Calendar
    """
    team_details = models.ForeignKey(
        TeamDetails, on_delete=models.CASCADE, blank=True, null=True)

    day = models.ForeignKey(
        DayofWeek, on_delete=models.CASCADE, blank=True, null=True)

    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="Deleted Status")

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.team_details, self.day)

    class Meta:
        verbose_name_plural = "Team Calendar"
        db_table = "team_calendar"


class BookMatch(models.Model):
    """
    Player Match Booking
    """
    MATCH_STATUS_CHOICE = (
        ("Upcoming", "Upcoming"),
        ("Completed", "Completed"),
        ("Cancelled", "Cancelled")
    )
    # STATUS_CHOICE = (
    #     ("Team", "Team"),
    #     ("Individual", "Individual")
    # )

    venue_field = models.ForeignKey(
        VenueField, on_delete=models.CASCADE, blank=True, null=True)
    schedule_slot = models.ManyToManyField(
        Schedule_Slot, blank=True)
    date = models.DateField(blank=True, null=True, help_text="Date")

    # services = models.ManyToManyField(
    #     VenueServiceCostQuantity, blank=True, null=True)
    services_cost = models.IntegerField(default=0, blank=True, null=True)

    total_cost = models.IntegerField(default=0, blank=True, null=True)
    my_team = models.ForeignKey(
        TeamDetails, on_delete=models.CASCADE, blank=True, null=True, related_name='my_team')
    opponent_team = models.ForeignKey(
        TeamDetails, on_delete=models.CASCADE, blank=True, null=True, related_name='opponent_team')
    # opponent_player = models.ForeignKey(
    #     Player, on_delete=models.CASCADE, blank=True, null=True, related_name='opponent_player')
    status = models.CharField(max_length=20, choices=MATCH_STATUS_CHOICE,
                              blank=True, null=True, help_text="Status of the match")
    # booking_type = models.CharField(max_length=20, choices=STATUS_CHOICE,
    #                                 blank=True, null=True, help_text="Status of the match")
    match_result = models.NullBooleanField(
        blank=True, help_text="true for win/false for loose")
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    created_by = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.id, self.venue_field,)

    # def save(self, *args, **kwargs):
    #     self.total_cost = self.total_cost + self.services_cost
    #     super(BookMatch, self).save(*args, **kwargs)

    class Meta:
        verbose_name_plural = "Team Book Match"
        db_table = "book_match"


class PlayerInvitation(models.Model):
    """
    All Player Invitations
    """
    CATEGORY_CHOICES = (
        ("Play Match", "Play Match"),  # Book a match
        ("Join Team", "Join Team"),  # Create Team
        ("Find Opponent", "Find Opponent"),  # Find Opponent
        ("Team Request", "Team Request"),  # Send invite to join team
        ("Team Match", "Team Match")  # Send Match request to team
    )
    STATUS_CHOICE = (
        ("Pending", "Pending"),
        ("Approved", "Approved"),
        ("Declined", "Declined")
    )

    recipient = models.CharField(
        max_length=100, blank=True, null=True, help_text="Recipient Email")
    category = models.CharField(choices=CATEGORY_CHOICES, max_length=20,
                                blank=True, null=True, help_text="Invitation category")
    status = models.CharField(max_length=20, choices=STATUS_CHOICE,
                              default=STATUS_CHOICE[0][0], blank=True, null=True, help_text="Status of the player")
    sender = models.CharField(
        max_length=100, blank=True, null=True, help_text="Recipient Email")
    created_by = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return "{}-{}-{}".format(self.id, self.recipient, self.sender)

    class Meta:
        verbose_name_plural = "Players Invitation"
        db_table = "player_invitation"


class VenueListPlayer(models.Model):
    venue_list = models.ForeignKey(Venue, on_delete=models.CASCADE)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE)

    def __str__(self):
        return "{}".format(self.venue_list)

    class Meta:
        verbose_name_plural = "VenueListPlayers"
        db_table = 'VenueListPlayer'


class VenuePaidServiceQuantity(models.Model):
    """ Venue Paid Service Quantity """

    book_match = models.ForeignKey(
        BookMatch, on_delete=models.CASCADE, blank=True, null=True)

    venue_service_cost = models.ForeignKey(
        VenueServiceCost, on_delete=models.CASCADE, blank=True, null=True)

    quantity = models.IntegerField(blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")

    def __str__(self):
        return "{}-{}".format(self.quantity, self.venue_service_cost)

    class Meta:
        verbose_name_plural = "Venue Paid Service Quantity"
        db_table = 'venue_paid_service_quantity'


class VenueFieldServiceQuantity(models.Model):
    """ Venue Filed Service Quantity """

    book_match = models.ForeignKey(
        BookMatch, on_delete=models.CASCADE, blank=True, null=True)

    venue_field_service = models.ForeignKey(
        VenueFieldService, on_delete=models.CASCADE, blank=True, null=True)

    quantity = models.IntegerField(blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")

    def __str__(self):
        return "{}-{}".format(self.quantity, self.venue_field_service)

    class Meta:
        verbose_name_plural = "Venue Field Service Quantity"
        db_table = 'venue_field_service_quantity'


class PlayerInterest(models.Model):
    """
    Player interests
    """
    interest_sports = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")

    def __str__(self):
        return "{}-{}".format(self.interest_sports, self.player)

    class Meta:
        verbose_name_plural = "Player Interests"
        db_table = 'player_interest'


class PlayerOpponentSearchInterest(models.Model):
    """
    Player interests
    """
    interested_for_opponent = models.BooleanField(
        default=False, blank=True, null=True)
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")

    def __str__(self):
        return "{}-{}".format(self.interested_for_opponent, self.player)

    class Meta:
        verbose_name_plural = "Player Opponent Interest"
        db_table = 'player_opponent_interest'


class PlayerNotifyContactCronScheduler(models.Model):
    """
        Schedule Email Crons
    """
    BACKGROUND_STATUS = (
        ("HOLD", 'HOLD'),
        ('RUNNING', 'RUNNING'),
        ('SUCCESS', 'SUCCESS'),
        ('FAILED', "FAILED")
    )

    to_player = models.ForeignKey(
        Player, on_delete=models.CASCADE, null=True, blank=True, help_text="Player")
    from_player = models.ForeignKey(Player, on_delete=models.CASCADE,
                                    null=True, blank=True, help_text="Player", related_name="from_player")
    sent_date = models.DateField(
        blank=True, null=True, help_text="Mail Sent date")
    status = models.CharField(max_length=100, choices=BACKGROUND_STATUS,
                              default=BACKGROUND_STATUS[0][0], help_text="Status")
    message = models.CharField(max_length=200,
                               blank=True, null=True, help_text="message")
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Schedule Slot")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.from_player, self.sent_date, self.status)

    class Meta:
        verbose_name_plural = "Player Notify Contacts"
        db_table = "player_notify_contacts"


class PlayerNotifyContactAvailabilityDates(models.Model):
    """
    Player Avaialbility dates
    """
    player_notify_contact = models.ForeignKey(
        PlayerNotifyContactCronScheduler, on_delete=models.CASCADE, blank=True, null=True)
    date = models.DateField(blank=True, null=True)
    start_time = models.TimeField(blank=True, null=True)
    day = models.CharField(max_length=100, blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)
    sport = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.date, self.start_time, self.day)

    class Meta:
        verbose_name_plural = "Player Notify Availability Dates"
        db_table = "player_notify_availability_dates"


class PlayerMyContactsPlatform(models.Model):
    """
    Player MyContacts Platform
    """
    STATUS_CHOICE = (
        ("Pending", "Pending"),
        ("Approved", "Approved"),
        ("Declined", "Declined")
    )
    email = models.CharField(max_length=100, null=True, help_text="user email")
    status = models.CharField(max_length=20, choices=STATUS_CHOICE,
                              default=STATUS_CHOICE[0][0], blank=True, null=True, help_text="Status")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")

    def __str__(self):
        return "{}-{}".format(self.email, self.status)

    class Meta:
        verbose_name_plural = "Player MyContacts Platform"
        db_table = 'player_my_contacts_platform'
