import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.db.models import Q
from django.urls import reverse
from django.db.models import Sum, Min
from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from players.serializers import *
from message.models import Notification
from venue_owners.serializers import Schedule_SlotSerializer, Schedule_SlotSummarySerializer
from players.models import Player, PlayerAvailability
from venue_owners.models import Schedule, Schedule_Slot, VenueField


class NotifyContactsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = NotifyContactsSerializer
    
    def post(self, request, format=None):
        """ 
        Notify Contacts API View
        """ 
        response = {}
        responsedata = []
        player_obj = Player.objects.filter(user=request.user,is_deleted=False)
        if player_obj.exists():
            player_obj = player_obj.last()
        else:
            raise Exception("Unauthorized")
        serializers = self.serializer_class(data=request.data, context={'player': player_obj})
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response['message'] = "Email will be sent to the selected contacts"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)



class GetContactsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FriendlistSerializer

    def get(self, request, format=None):
        """
        Get Contacts
        """
        try:
            response = {}
            responsedata = []
            # Check for player
            player_obj = Player.objects.filter(user=request.user,is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                raise Exception("Unauthorized")
            data = FriendList.objects.filter(user=player_obj)
            serializers = self.serializer_class(data, many=True)
            response["result"] = 1
            response['data'] = serializers.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as exec:
            response["result"] = 0
            response["errors"] = [exec]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class POSTAvailabilityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerAvailabilitySerializer

    def get(self, request, format=None):

        response = {}
        responsedata = []
        user = User.objects.get(username=request.user).id
        data = PlayerAvailability.objects.filter(
            player=Player.objects.get(user=user), is_deleted=False)
        serializers = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializers.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):

        response = {}
        responsedata = []
        if 'id' in request.data:
            if PlayerAvailability.objects.filter(id=request.data['id']).exists:
                serializers = self.serializer_class(instance=PlayerAvailability.objects.get(
                    id=request.data['id'], is_deleted=False), data=request.data)
            else:
                responsedata.append("No Record with the given ID")
                response["result"] = 0
                response['errors'] = responsedata
                response['data'] = serializers.data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        else:
            serializers = self.serializer_class(
                data=request.data, context={'user': request.user})
        if serializers.is_valid():
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['errors'] = responsedata
            response['data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetFieldPriceAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetFieldPriceSerializer
    slot_serializer_class = Schedule_SlotSerializer

    def post(self, request, format=None):
        """ 
        Get Field Price for put your availability
        """
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            booking_date = serializers.validated_data.get('booking_date', None)
            start_hour = serializers.validated_data.get('start_hour', None)
            end_hour = serializers.validated_data.get('end_hour', None)

            venue_field_obj = VenueField.objects.filter(
                id=serializers.validated_data['field_id'], is_deleted=False)

            if venue_field_obj.exists():
                venue_field_obj = venue_field_obj.last()
            else:
                response["result"] = 0
                response["errors"] = ["No SUch Field Found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # GEt Schedule
            schedule_obj = Schedule.objects.filter(Q(start_date__lte=booking_date, end_date__gte=booking_date, template__start_hour__lte=start_hour, template__end_hour__gte=start_hour) | Q(
                start_date__lte=booking_date, end_date__gte=booking_date, template__start_hour__gte=end_hour, template__end_hour__lte=end_hour))
            slots = 0
            for i in schedule_obj:
                if Schedule_Slot.objects.filter(schedule_id=i, date=booking_date).exists():
                    schedule_slots = Schedule_Slot.objects.filter(Q(schedule_id=i, date=booking_date, is_available=True, start_hour__lte=start_hour) | Q(
                        schedule_id=i, date=booking_date, is_available=True, end_hour__gte=start_hour)).aggregate(Min('cost'))

            response["result"] = 1
            response["starting_price"] = schedule_slots['cost__min']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteAvailabilityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerAvailabilityDeleteSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            data = PlayerAvailability.objects.filter(
                id=serializers.validated_data['id']).first()
            data.is_deleted = True
            data.save(update_fields=['is_deleted'])
            response['result'] = 1
            response['errors'] = ["Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerAvailabilityNotifyContactAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerNotifyMyContactsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data)

        if serializers.is_valid():

            notification_contact = [Notification(
                recipient=Player.objects.filter(id=i).last().user,
                category="Play Match",
                subject="Match Invitation",
                content_object=Player.objects.filter(id=i).last(),
                body="You havebeen invited to play a match."
            )
                for i in serializers.validated_data['id']]
            Notification.objects.bulk_create(
                notification_contact)

            response["result"] = 1
            response['message'] = "Invitation Sent Successfully"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
