from django.db import models

from django.core.management.base import BaseCommand, CommandError

import datetime
from players.models import Player

class DuplicateTaskException(Exception):
    pass


class TaskAlreadyInProgress(Exception):
    pass


class Task(models.Model):

    MAX_EXECUTION_TIME = 200*60  # secs

    UNPROCESSED = 0
    RUNNING = 1
    SUCCESSFUL = 2
    FAILED = 3

    STATE_CHOICES = (
        (UNPROCESSED, 'Unprocessed'),
        (RUNNING, 'Running'),
        (SUCCESSFUL, 'Success'),
        (FAILED, 'Failed'),
    )

    TASK_TYPES = (
        ("Campaign Mail", 'Campaign Mail'),
    )

    name = models.CharField(max_length=255)
    command = models.CharField(max_length=255, blank=True, null=True)
    argument_list = models.CharField(max_length=255, null=True, blank=True)

    # notification   = models.ForeignKey(Notification, blank=True, null=True)

    state = models.IntegerField(choices=STATE_CHOICES, default=UNPROCESSED)
    priority = models.IntegerField(default=1)
    progress = models.TextField(null=True)
    task_type = models.CharField(
        choices=TASK_TYPES, max_length=255, null=True, blank=True)

    scheduled_time = models.DateTimeField(blank=True, null=True)
    ran = models.BooleanField(default=False)
    ran_at = models.DateTimeField(null=True, blank=True)
    finished_at = models.DateTimeField(null=True, blank=True)
    player  = models.ForeignKey(Player, on_delete=models.SET_NULL, blank=True, null=True)

    cancel = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = "Tasks"
        db_table = 'tasks'

    @staticmethod
    def get_next_task_in_queue():
        """ Return the next task to be run """
        if not Task.get_tasks_in_progress():
            result = Task.objects.filter(state=Task.UNPROCESSED, ran=False, cancel=False, scheduled_time__lt=datetime.datetime.now(
                datetime.timezone.utc)).order_by('-priority', 'id')
            if result:
                return result
            else:
                raise Task.DoesNotExist('No Pending tasks found')
        else:
            raise TaskAlreadyInProgress('Error! Task already in progress')

    @staticmethod
    def get_tasks_in_progress():
        """ Return any task/tasks still in running state """
        query = Task.objects.filter(state=Task.RUNNING)
        return query


#####  ==========================================  ######

def progress_callback(task_id):
    """ Update the progress fields of a Task with id task_id """
    def update_task(progress, state, ran_at=None, finished_at=None, ran=False, exception=None):
        task = Task.objects.get(id=task_id)
        task.progress = progress
        task.state = state
        if ran_at:
            task.ran_at = ran_at
        if finished_at:
            task.finished_at = finished_at
        if ran:
            task.ran = ran
        task.save()

    return update_task


class TaskCommand(BaseCommand):

    def add_callbacks(self, options):
        try:
            self.progress_callback = options['progress_callback']
            self.task_id = options['task_id']
            del options['progress_callback']
            del options['task_id']
        except KeyError:
            def progress_callback(progress, state, run_at=None, finished_at=None, exception=None):
                print(progress)
            self.progress_callback = progress_callback
