from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import *
import datetime
from venues.models import *


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class VenueOwner(models.Model):
    """ Venue Owner User details reside here """
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    user = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    first_name = models.CharField(
        max_length=100, null=True, help_text="user first name")
    middle_name = models.CharField(
        max_length=100, null=True, help_text="user middle name", blank=True)
    last_name = models.CharField(
        max_length=100, null=True, help_text="user last name", blank=True)

    email = models.CharField(max_length=100, null=True, help_text="user email")
    new_email = models.CharField(
        max_length=100, null=True, blank=True, help_text="user new email")
    mobile = models.CharField(max_length=15, null=True,
                              blank=True, help_text="user mobile number")
    # city = models.CharField(max_length=50, blank=True,null=True, help_text="city")

    # country = models.CharField(max_length=50, blank=True, null=True, help_text="country")
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)
    payment_method = models.ForeignKey(
        PaymentMethod, on_delete=models.CASCADE, blank=True, null=True)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True, help_text="vat id")
    vat_exemption_order = models.ForeignKey(
        VatExemptionOrder, on_delete=models.CASCADE, blank=True, null=True, help_text="vat exemption order id")
    profession = models.CharField(max_length=50, blank=True,
                                  null=True, help_text="profession")
    street = models.CharField(max_length=50, blank=True,
                              null=True, help_text="street")
    venue_number = models.CharField(max_length=50, blank=True,
                                    null=True, help_text="venue_number")
    pincode = models.CharField(max_length=50, blank=True,
                               null=True, help_text="pincode")
    # mobile     = models.CharField(max_length=15, null=True, help_text="user mobile number", unique=True)
    new_mobile = models.CharField(
        max_length=15, null=True, blank=True, help_text="user new mobile number")
    gender = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES,
                              help_text="M - Male, F - Female, O - Other")

    telephone = models.CharField(max_length=15, null=True, blank=True)
    telephone2 = models.CharField(max_length=15, null=True, blank=True)
    # telephone3 = models.CharField(max_length=15, null=True, blank = True)
    user_code = models.CharField(max_length=6, null=True, blank=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")
    is_email_verified = models.BooleanField(default=False)

    # created_by = models.ForeignKey(InternalUser, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.pk, self.mobile, self.first_name)

    class Meta:
        unique_together = ('user', 'email')
        verbose_name_plural = "Venue Owners"
        db_table = 'venue_owner'


class PriceListStep1(models.Model):
    """
    Price List Step 1 data
    """
    list_name = models.CharField(
        max_length=100, null=True, help_text="Price List Name")
    interval = models.IntegerField(help_text="Interval in Minutes", null=True)

    gap_period = models.IntegerField(
        help_text="Gap Period in Minutes", null=True)
    fields = models.ManyToManyField(VenueField, blank=True)
    start_hour = models.TimeField(blank=True, null=True)

    end_hour = models.TimeField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate PriceList")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.list_name, self.interval, self.gap_period)

    class Meta:
        verbose_name_plural = "Price List 1"
        db_table = 'price_list1'


class PriceListStep2(models.Model):
    """
    Price List Step 2 data
    """
    price_list1 = models.ForeignKey(PriceListStep1, on_delete=models.CASCADE)
    day = models.ForeignKey(DayofWeek, on_delete=models.CASCADE)

    start_hour = models.TimeField(blank=True, null=True)
    end_hour = models.TimeField(blank=True, null=True)
    price = models.IntegerField(blank=True, null=True, default=0)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate PriceList")

    def __str__(self):
        return "{}-{}-{}".format(self.price_list1, self.day, self.created_by)

    class Meta:
        verbose_name_plural = "Price List 2"
        db_table = 'price_list2'


class ScheduleTemplateStep1(models.Model):
    """
    Schedule Template Step 1 data
    """
    RANGE_CHOICE = (
        ("D", "Daily"),
        ("W", "Weekly"),
        ("M", "Monthly")
    )

    template_name = models.CharField(max_length=100, null=True, blank=True)

    start_hour = models.TimeField(
        blank=True, null=True, help_text="Template Start Hour")
    end_hour = models.TimeField(
        blank=True, null=True, help_text="Template End Hour")

    interval = models.IntegerField(help_text="Interval in Minutes", null=True)
    gap_period = models.IntegerField(
        help_text="Gap Period in Minutes", null=True)
    plan_range = models.CharField(
        max_length=1, choices=RANGE_CHOICE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate PriceList")

    def __str__(self):
        return "{}-{}-{}".format(self.template_name, self.start_hour, self.end_hour)

    class Meta:
        verbose_name_plural = "Schedule Template 1"
        db_table = 'schedule_template1'


class ScheduleTemplateSlot(models.Model):
    """
        Schedule Template Time Slots
    """
    template = models.ForeignKey(
        ScheduleTemplateStep1, on_delete=models.CASCADE)

    start_hour = models.TimeField(
        blank=True, null=True, help_text="Template Start Hour")
    end_hour = models.TimeField(
        blank=True, null=True, help_text="Template End Hour")

    is_available = models.BooleanField(null=True, default=True, blank=True)
    day = models.IntegerField(blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.template, self.start_hour, self.end_hour)

    class Meta:
        verbose_name_plural = "Schedule Template Slots"
        db_table = 'template_slots'


###############  SCHEDULE  #############################


class Schedule(models.Model):
    """
        Schedule data
    """

    schedule_name = models.CharField(
        max_length=100, blank=True, null=True, help_text="Schedule Name")
    field = models.ForeignKey(VenueField, on_delete=models.CASCADE,
                              blank=True, null=True, help_text="Venue Field")

    template = models.ForeignKey(ScheduleTemplateStep1, on_delete=models.CASCADE,
                                 null=True, blank=True, help_text="Schedule Template Model")
    pricelist = models.ForeignKey(
        PriceListStep1, on_delete=models.CASCADE, null=True, blank=True, help_text="Price List Model")

    start_date = models.DateField(
        blank=True, null=True, help_text="Start Date")
    end_date = models.DateField(blank=True, null=True, help_text="End Date")

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Schedule")

    def __str__(self):
        return "{}-{}-{}".format(self.schedule_name, self.template, self.start_date)

    class Meta:
        verbose_name_plural = "Schedule"
        db_table = "schedule"


class Schedule_Slot(models.Model):
    """
        Schedule Slots
    """

    schedule_id = models.ForeignKey(
        Schedule, on_delete=models.CASCADE, null=True, blank=True, help_text="Schedule ID")
    start_hour = models.TimeField(
        blank=True, null=True, help_text="Starting Time")

    end_hour = models.TimeField(blank=True, null=True, help_text="End Time")
    date = models.DateField(blank=True, null=True, help_text="Date")

    day = models.ForeignKey(DayofWeek, on_delete=models.CASCADE,
                            null=True, blank=True, help_text="Day of week")
    is_available = models.BooleanField(null=True, default=True, blank=True)
    cost = models.FloatField(blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Schedule Slot")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}-{}".format(self.schedule_id, self.start_hour, self.end_hour, self.id, self.date)

    class Meta:
        verbose_name_plural = "Schedule Slots"
        db_table = "schedule_slot"


class PackageTypes(models.Model):
    """
Package Type
"""
    description = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.description)

    class Meta:
        verbose_name_plural = 'Package Types'
        db_table = 'package_types'


class PackageDurations(models.Model):
    """
Package Durations
"""
    description = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.description)

    class Meta:
        verbose_name_plural = 'Package Durations'
        db_table = 'package_durations'


class Packages(models.Model):
    """
    Packages data
    """
    # PACKAGE_CHOICE = (
    #     ("S", "Subscription"),
    #     ("L", "Lessons"),
    #     ("B", "Bookings Package")
    # )
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )

    package_image = models.ImageField(
        upload_to="Package", blank=True, null=True)

    name = models.CharField(
        max_length=100, blank=True, null=True, help_text="Package Name")
    package_description = models.CharField(
        max_length=600, blank=True, null=True, help_text="Package Description")
    sport_relivance = models.ForeignKey(
        Sport, on_delete=models.CASCADE, null=True, blank=True, help_text="Sports ID")

    level = models.ManyToManyField(TeamLevel, blank=True)
    package_type = models.ForeignKey(
        PackageTypes, on_delete=models.CASCADE, null=True, blank=True, help_text="Package type Id")
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True,
        help_text="Is Verified")
    package_duration = models.ForeignKey(
        PackageDurations, on_delete=models.CASCADE, null=True, blank=True,  help_text="Duration Id")
    number_sessions = models.IntegerField(
        help_text="Number of sessions", blank=True, null=True)
    session_duration = models.IntegerField(
        help_text="Session duration in Minutes", blank=True, null=True)
    price = models.CharField(max_length=100, blank=True, null=True)
    package_quantity = models.IntegerField(blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False)

    def __str__(self):
        return "{}-{}-{}-{}-{}-{}".format(self.name, self.package_description, self.package_quantity, self.package_type, self.sport_relivance,
                                          self.price)

    class Meta:
        verbose_name_plural = "Packages"
        db_table = 'packages'


class Schedule_Email_Cron(models.Model):
    """
        Schedule Email Crons
    """
    BACKGROUND_STATUS = (
        ("HOLD", 'HOLD'),
        ('RUNNING', 'RUNNING'),
        ('SUCCESS', 'SUCCESS'),
        ('FAILED', "FAILED")
    )

    schedule_id = models.ForeignKey(
        Schedule, on_delete=models.CASCADE, null=True, blank=True, help_text="Schedule ID")
    schedule_end_date = models.DateField(
        blank=True, null=True, help_text="end date")
    run_date = models.DateField(blank=True, null=True, help_text="run date")
    status = models.CharField(max_length=100, choices=BACKGROUND_STATUS,
                              default=BACKGROUND_STATUS[0][0], help_text="Status")

    message = models.CharField(max_length=200,
                               blank=True, null=True, help_text="message")

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Schedule Slot")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.schedule_id, self.schedule_end_date, self.status)

    class Meta:
        verbose_name_plural = "Schedule Email Crons"
        db_table = "schedule_email_crons"


class Time_Slot_Price(models.Model):
    """
        Time price Slots
    """

    schedule_id = models.ForeignKey(
        Schedule, on_delete=models.CASCADE, null=True, blank=True, help_text="Schedule ID")
    start_hour = models.TimeField(
        blank=True, null=True, help_text="Starting Time")

    end_hour = models.TimeField(blank=True, null=True, help_text="End Time")
    cost = models.FloatField(blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Schedule Slot")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.schedule_id, self.start_hour, self.end_hour)

    class Meta:
        verbose_name_plural = "Time Slots Price"
        db_table = "time_slot_price"


class BookingCancellationRequest(models.Model):
    """
Booking Cancellation Request
    """
    STATUS_CHOICES = (
        ('A', 'Approve'),
        ('R', 'Rejected'),
        ('N', 'New')
    )
    cancellation_reason = models.ForeignKey(
        BookingCancellation, on_delete=models.CASCADE, blank=True, null=True)
    venue_field = models.ForeignKey(
        VenueField, on_delete=models.CASCADE, blank=True, null=True)
    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)
    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)
    start_date = models.DateField(
        blank=True, null=True, help_text="Start Date")
    end_date = models.DateField(blank=True, null=True, help_text="End Date")
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True,
        help_text="Is Verified")
    comments = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{},{}".format(self.cancellation_reason, self.start_date)

    class Meta:
        verbose_name_plural = 'Booking Cancellation Request'
        db_table = 'Booking_Cancellation_Request'


class Schedule_Parameter(models.Model):
    """
        Schedule Parameters
    """
    schedule_edit = models.IntegerField(blank=True, null=True)
    schedule_delete = models.IntegerField(blank=True, null=True)
    schedule_create = models.IntegerField(blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.schedule_edit, self.schedule_delete, self.schedule_create)

    class Meta:
        verbose_name_plural = "Schedule Parameter"
        db_table = "schedule_parameter"


class RequestBookingCancellation(models.Model):
    """
Booking Cancellation Request
    """
    STATUS_CHOICES = (
        ('A', 'Approve'),
        ('R', 'Rejected'),
        ('N', 'New')
    )
    cancellation_reason = models.ForeignKey(
        BookingCancellation, on_delete=models.CASCADE, blank=True, null=True)
    venue_field = models.ManyToManyField(VenueField, blank=True)
    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)
    start_date = models.DateField(
        blank=True, null=True, help_text="Start Date")
    end_date = models.DateField(blank=True, null=True, help_text="End Date")
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True,
        help_text="Is Verified")
    comments = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{},{},{},{}".format(self.cancellation_reason, self.venue_field, self.start_date, self.created_by)

    class Meta:
        verbose_name_plural = 'Request Booking Cancellation'
        db_table = 'request_booking_cancellation'


class Testing_data(models.Model):
    """
Static Testing
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Testing Data'
        db_table = 'testing_data'
